//
//  ARGCamera.swift
//  ARGearSample
//
//  Created by Jaecheol Kim on 2019/11/07.
//  Copyright © 2019 Seerslab. All rights reserved.
//

import Foundation
import AVFoundation

//typealias SampleBufferHandler = (AVCaptureOutput, CMSampleBuffer, AVCaptureConnection) -> Void

class ARGCamera2: NSObject {
    private let kCameraZNear = CGFloat(0.05)
    private let kCameraZFar = CGFloat(100)
    
    var captureDevice: AVCaptureDevice?
    var captureSession: AVCaptureSession?
    
    var sampleBufferHandler: SampleBufferHandler?
    
    override init() {
        super.init()
        setupCamera()
    }

    private func setupCamera() {          
        guard
            let device = AVCaptureDevice.default(.builtInWideAngleCamera, for: .video, position: .front),
            let input = try? AVCaptureDeviceInput(device: device)
            else {
            fatalError("Failed to create capture device from front camera.")
        }
        let output = AVCaptureVideoDataOutput()
        output.videoSettings = [kCVPixelBufferPixelFormatTypeKey as String: kCVPixelFormatType_32BGRA]
        output.setSampleBufferDelegate(self, queue: DispatchQueue.global(qos: .userInteractive))

        let session = AVCaptureSession()
        session.sessionPreset = .iFrame1280x720
        session.addInput(input)
        session.addOutput(output)
        captureSession = session
        captureDevice = device
    }

    private func getVideoPermission(permissionHandler: @escaping (Bool) -> ()) {
        switch AVCaptureDevice.authorizationStatus(for: .video) {
        case .authorized:
          permissionHandler(true)
        case .notDetermined:
          AVCaptureDevice.requestAccess(for: .video, completionHandler: permissionHandler)
        default:
          permissionHandler(false)
        }
    }
    
    func startCamera() {
        getVideoPermission(permissionHandler: { granted in
          guard granted else {
            fatalError("Permission not granted to use camera.")
          }
          self.captureSession?.startRunning()
        })
    }
}

// MARK: - Camera delegate
extension ARGCamera2 : AVCaptureVideoDataOutputSampleBufferDelegate {
    func captureOutput(_ output: AVCaptureOutput, didDrop sampleBuffer: CMSampleBuffer, from connection: AVCaptureConnection) {

    }

    // synchronizer
    func captureOutput(_ output: AVCaptureOutput, didOutput sampleBuffer: CMSampleBuffer, from connection: AVCaptureConnection) {
        if let sampleBufferHandler = sampleBufferHandler
        {
            sampleBufferHandler(output, sampleBuffer, connection)
        }
    }
    
}
