//
//  ViewController.swift
//  ARGearSample
//
//  Created by Jaecheol Kim on 2019/10/28.
//  Copyright © 2019 Seerslab. All rights reserved.
//

import UIKit
import CoreMedia
import SceneKit
import AVFoundation
import ARGear

public final class ViewController: UIViewController {
    
    // MARK: - ARGearSDK properties
    private var argConfig : ARGConfig?
    private var argSession : ARGSession?
    private var currentFaceFrame: ARGFrame?
    private var nextFaceFrame: ARGFrame?
    
    // MARK: - Camera & Scene properties
    private let serialQueue = DispatchQueue(label: "serialQueue")
    private var currentCamera: CameraDeviceWithPosition = .front
    
    private var arCamera: ARGCamera!
    private var arScene: ARGScene!
    private var arMedia: ARGMedia!
    
    private lazy var cameraPreviewCALayer = CALayer()

    // MARK: - viewDidLoad
    override public func viewDidLoad() {
      super.viewDidLoad()
        setupConfig()
        setupScene()
        setupCamera()
    }
    // MARK: - viewWillAppear
    override public func viewWillAppear(_ animated: Bool) {
        super.viewWillAppear(animated)
        runARGSession()
    }

    // MARK: - ARGearSDK setupConfig
    private func setupConfig() {
        do {
            let config = ARGConfig(
                apiURL: "https://apis.argear.io/api/v3/",
                apiKey: "48cdd456621ebddb5283f516",
                secretKey: "e0ea29ce12178111d2a5e6093a6e048a2a6248b921f092f68c93992ca29a45ee",
                authKey: "U2FsdGVkX19DguB6YrrW+9R193+Jsekcvt38I7dROnYZ7NkmCuCff843TkNC4CWTgYVplAnXjirwOxK/uAGVqQ=="
            )

            argSession = try ARGSession(argConfig: config)
            argSession?.delegate = self
            
        } catch let error as NSError {
            NSLog("Failed to initialize ARGear Session with error: %@", error.description)
        } catch let exception as NSException {
            NSLog("Exception to initialize ARGear Session with error: %@", exception.description)
        }
    }

    // MARK: - setupScene
    private func setupScene() {
        arScene = ARGScene(viewContainer: view)

        arScene.sceneRenderUpdateAtTimeHandler = { [weak self] renderer, time in
            guard let self = self else { return }
            self.refreshARFrame()
        }

        arScene.sceneRenderDidRenderSceneHandler = { [weak self] renderer, scene, time in
            guard let self = self else { return }
            self.drawARCameraPreview()
        }

        cameraPreviewCALayer.contentsGravity = .center
        cameraPreviewCALayer.frame = arScene.sceneView.bounds
        view.layer.insertSublayer(cameraPreviewCALayer, at: 0)
    }
    
    // MARK: - setupCamera
    private func setupCamera() {
        arCamera = ARGCamera(camera: currentCamera)
        
        arCamera.sampleBufferHandler = { [weak self] output, sampleBuffer, connection in
            guard let self = self else { return }
            
            self.serialQueue.async {

                self.argSession?.update(sampleBuffer, from: connection)
            }
        }
        
        arCamera.dataBufferHandler = { [weak self] sampleBuffer, metadataObjects in
            guard let self = self else { return }

            self.serialQueue.async {
                if CMSampleBufferIsValid(sampleBuffer) {
                    self.argSession?.update(sampleBuffer, from: self.arCamera.cameraConnection!)
                }
                if metadataObjects != nil {
                    self.argSession?.update(metadataObjects as! [AVMetadataObject], from: self.arCamera.cameraConnection!)
                }
            }
        }
        
        arCamera.startCamera()
        
        arMedia = ARGMedia()
        arMedia.setVideoConnection(arCamera.cameraConnection!)
    }
    
    // MARK: - ARGearSDK Handling
    private func refreshARFrame() {
        guard self.nextFaceFrame != nil && self.nextFaceFrame != self.currentFaceFrame else { return }
        self.currentFaceFrame = self.nextFaceFrame
    }
    
    private func drawARCameraPreview() {
        guard
            let frame = self.currentFaceFrame,
            let pixelBuffer = frame.renderedPixelBuffer
        else { return }
        
        CATransaction.begin()
        CATransaction.setAnimationDuration(0)

        self.cameraPreviewCALayer.contents = pixelBuffer

        let frameSize = self.cameraPreviewCALayer.bounds.size
        self.cameraPreviewCALayer.setAffineTransform(
            frame.displayTransform(
            forFrameSize: frameSize,
            presentationOrientation: .portrait,
            mirrored: true)
        )
        CATransaction.commit()
    }

    private func runARGSession() {
        argSession?.inferenceDebugOption = .optionDebugFaceLandmark2D
        let inferenceFeature: ARGInferenceFeature? = [.faceTracking , .faceLandmark2D]

        argSession?.run(withInferenceConfig: inferenceFeature!)
    }

}

// MARK: - ARGearSDK ARGSession delegate
extension ViewController : ARGSessionDelegate {

      public func didUpdate(_ arFrame: ARGFrame) {
        for face in arFrame.faces.faceList {
           if face.isValid {
//            NSLog("landmarkcount = %d", face.landmark.landmarkCount)
            
            // get face information(landmarkCoordinates , rotation_matrix, translation_vector)
            // let landmarkcount = face.landmark.landmarkCount
            // let landmarkCoordinates = face.landmark.landmarkCoordinates
            // let rotation_matrix = face.rotation_matrix
            // let translation_vector = face.translation_vector
           }
        }
        
        nextFaceFrame = arFrame
    }
    
}
